import type { MetaReducer } from '@ngrx/store';
import type { QueryKeys } from '@reduxjs/toolkit/dist/query/core/apiState';
import type { PrefetchOptions } from '@reduxjs/toolkit/dist/query/core/module';
import type { QueryArgFrom } from '@reduxjs/toolkit/dist/query/endpointDefinitions';
import type { BaseQueryFn, EndpointDefinitions, Module, MutationDefinition, QueryDefinition } from '@reduxjs/toolkit/query';
import { dispatch, getState as getStateFromStore, select } from './thunk.service';
import { HooksWithUniqueNames, MutationHooks, QueryHooks } from './types';
export declare const angularHooksModuleName: unique symbol;
export type AngularHooksModule = typeof angularHooksModuleName;
declare module '@reduxjs/toolkit/query' {
    interface ApiModules<BaseQuery extends BaseQueryFn, Definitions extends EndpointDefinitions, ReducerPath extends string, TagTypes extends string> {
        [angularHooksModuleName]: {
            /**
             *  Endpoints based on the input endpoints provided to `createApi`, containing `select`, `hooks` and `action matchers`.
             */
            endpoints: {
                [K in keyof Definitions]: Definitions[K] extends QueryDefinition<any, any, any, any, any> ? QueryHooks<Definitions[K]> : Definitions[K] extends MutationDefinition<any, any, any, any, any> ? MutationHooks<Definitions[K]> : never;
            };
            /**
             * A hook that accepts a string endpoint name, and provides a callback that when called,
             * pre-fetches the data for that endpoint.
             */
            usePrefetch<EndpointName extends QueryKeys<Definitions>>(endpointName: EndpointName, options?: PrefetchOptions): (arg: QueryArgFrom<Definitions[EndpointName]>, options?: PrefetchOptions) => void;
        } & HooksWithUniqueNames<Definitions> & {
            metareducer: MetaReducer<any>;
        };
    }
}
export interface AngularHooksModuleOptions {
    /**
     * The version of the `useDispatch` hook to be used
     */
    useDispatch?: typeof dispatch;
    /**
     * The version of the `getState` to be used
     */
    getState?: typeof getStateFromStore;
    /**
     * The version of the `useSelector` hook to be used
     */
    useSelector?: typeof select;
}
/**
 * Creates a module that generates hooks from endpoints, for use with `buildCreateApi`.
 *
 * @returns A module for use with `buildCreateApi`
 */
export declare const angularHooksModule: ({ useDispatch, useSelector, getState, }?: AngularHooksModuleOptions) => Module<AngularHooksModule>;
export declare const createApi: import("@reduxjs/toolkit/query").CreateApi<typeof import("@reduxjs/toolkit/dist/query/core/module").coreModuleName | typeof angularHooksModuleName>;
